"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.Preset = exports.DEFAULT_ANIMATION_CONFIG = void 0;
var _react = _interopRequireWildcard(require("react"));
var _reactNative = require("react-native");
var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));
var _reactNativeGestureHandler = require("react-native-gesture-handler");
var _pageInterpolators = require("./pageInterpolators");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
let Preset = exports.Preset = /*#__PURE__*/function (Preset) {
  Preset["SLIDE"] = "slide";
  Preset["CUBE"] = "cube";
  Preset["STACK"] = "stack";
  Preset["TURN_IN"] = "turn-in";
  return Preset;
}({});
const PageInterpolators = {
  [Preset.SLIDE]: _pageInterpolators.pageInterpolatorSlide,
  [Preset.CUBE]: _pageInterpolators.pageInterpolatorCube,
  [Preset.STACK]: _pageInterpolators.pageInterpolatorStack,
  [Preset.TURN_IN]: _pageInterpolators.pageInterpolatorTurnIn
};
const DEFAULT_ANIMATION_CONFIG = exports.DEFAULT_ANIMATION_CONFIG = {
  damping: 20,
  mass: 0.2,
  stiffness: 100,
  overshootClamping: false,
  restSpeedThreshold: 0.2,
  restDisplacementThreshold: 0.2
};
const EMPTY_SIMULTANEOUS_GESTURES = [];
const EMPTY_ANIMATION_CONFIG = {};
function InfinitePager(_ref, ref) {
  let {
    vertical = false,
    PageComponent,
    pageCallbackNode,
    onPageChange,
    pageBuffer = 1,
    style,
    pageWrapperStyle,
    minIndex = -Infinity,
    maxIndex = Infinity,
    simultaneousGestures = EMPTY_SIMULTANEOUS_GESTURES,
    gesturesDisabled,
    animationConfig = EMPTY_ANIMATION_CONFIG,
    renderPage,
    flingVelocity = 500,
    preset = Preset.SLIDE,
    pageInterpolator = PageInterpolators[preset],
    bouncePct = 0.0,
    debugTag = "",
    width,
    height,
    minDistance
  } = _ref;
  const orientation = vertical ? "vertical" : "horizontal";
  const pageWidth = (0, _reactNativeReanimated.useSharedValue)(width || 0);
  const pageHeight = (0, _reactNativeReanimated.useSharedValue)(height || 0);
  const pageSize = vertical ? pageHeight : pageWidth;
  const translateX = (0, _reactNativeReanimated.useSharedValue)(0);
  const translateY = (0, _reactNativeReanimated.useSharedValue)(0);
  const translate = vertical ? translateY : translateX;
  const [curIndex, setCurIndex] = (0, _react.useState)(0);
  const pageAnimInternal = (0, _reactNativeReanimated.useSharedValue)(0);
  const pageAnim = pageCallbackNode || pageAnimInternal;
  const {
    simultaneousGestures: parentGestures,
    activePagers,
    nestingDepth
  } = (0, _react.useContext)(InfinitePagerContext);
  const pagerId = `${orientation}:${nestingDepth}`;
  const pageInterpolatorRef = (0, _react.useRef)(pageInterpolator);
  pageInterpolatorRef.current = pageInterpolator;
  const curIndexRef = (0, _react.useRef)(curIndex);
  curIndexRef.current = curIndex;
  const animCfgRef = (0, _react.useRef)(animationConfig);
  animCfgRef.current = animationConfig;
  const gesturesDisabledAnim = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return !!gesturesDisabled;
  }, [gesturesDisabled]);
  const setPage = (0, _react.useCallback)(function (index) {
    let options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    const updatedTranslate = index * pageSize.value * -1;
    if (index < minIndex || index > maxIndex) return;
    if (options.animated) {
      const animCfg = {
        ...DEFAULT_ANIMATION_CONFIG,
        ...animCfgRef.current
      };
      translate.value = (0, _reactNativeReanimated.withSpring)(updatedTranslate, animCfg);
    } else {
      translate.value = updatedTranslate;
    }
  }, [pageSize, translate, minIndex, maxIndex]);
  (0, _react.useImperativeHandle)(ref, () => ({
    setPage,
    incrementPage: options => {
      setPage(curIndexRef.current + 1, options);
    },
    decrementPage: options => {
      setPage(curIndexRef.current - 1, options);
    }
  }), [setPage]);
  const pageIndices = [...Array(pageBuffer * 2 + 1)].map((_, i) => {
    const bufferIndex = i - pageBuffer;
    return curIndex - bufferIndex;
  });
  (0, _reactNativeReanimated.useDerivedValue)(() => {
    if (pageSize.value) {
      pageAnim.value = translate.value / pageSize.value * -1;
    }
  }, [pageSize, pageAnim, translate]);
  function onPageChangeInternal(pg) {
    onPageChange === null || onPageChange === void 0 || onPageChange(pg);
    setCurIndex(pg);
  }
  (0, _reactNativeReanimated.useAnimatedReaction)(() => {
    return Math.round(pageAnim.value);
  }, (cur, prev) => {
    if (cur !== prev) {
      (0, _reactNativeReanimated.runOnJS)(onPageChangeInternal)(cur);
    }
  }, []);
  const startTranslate = (0, _reactNativeReanimated.useSharedValue)(0);
  const panGesture = (0, _react.useMemo)(() => _reactNativeGestureHandler.Gesture.Pan(), []);
  const minIndexAnim = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return minIndex;
  }, [minIndex]);
  const maxIndexAnim = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return maxIndex;
  }, [maxIndex]);
  const isMinIndex = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return curIndex <= minIndex;
  }, [curIndex, minIndex]);
  const isMaxIndex = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return curIndex >= maxIndex;
  }, [curIndex, maxIndex]);
  const isAtEdge = isMinIndex || isMaxIndex;
  const initTouchX = (0, _reactNativeReanimated.useSharedValue)(0);
  const initTouchY = (0, _reactNativeReanimated.useSharedValue)(0);
  const isGestureLocked = (0, _reactNativeReanimated.useDerivedValue)(() => {
    // Gesture goes to the most-nested active child of both orientations
    // All other pagers are locked
    const isDeepestInOrientation = activePagers.value.filter(v => {
      return v.split(":")[0] === orientation;
    }).every(v => {
      return Number(v.split(":")[1]) <= nestingDepth;
    });
    return activePagers.value.length && !isDeepestInOrientation;
  }, [activePagers, orientation]);
  panGesture.onBegin(evt => {
    "worklet";

    if (!isAtEdge) {
      const updated = activePagers.value.slice();
      updated.push(pagerId);
      activePagers.value = updated;
    }
    startTranslate.value = translate.value;
    initTouchX.value = evt.x;
    initTouchY.value = evt.y;
    if (debugTag) {
      console.log(`${debugTag} onBegin`, evt);
    }
  }).onTouchesMove((evt, mgr) => {
    "worklet";

    const mainTouch = evt.changedTouches[0];
    const evtVal = mainTouch[vertical ? "y" : "x"];
    const initTouch = vertical ? initTouchY.value : initTouchX.value;
    const evtTranslate = evtVal - initTouch;

    // const xAxisVal = mainTouch[vertical ? "x" : "y"]
    // const xAxisInitTouch = vertical ? initTouchY.value : initTouchX.value;
    // const xAxisTranslate = xAxisVal - xAxisInitTouch
    // const isSwipingCrossAxis = Math.abs(xAxisTranslate) > 10 && Math.abs(xAxisTranslate) > Math.abs(evtTranslate)

    const swipingPastEnd = isMinIndex.value && evtTranslate > 0 || isMaxIndex.value && evtTranslate < 0;
    const shouldFailSelf = !bouncePct && swipingPastEnd || isGestureLocked.value || gesturesDisabledAnim.value;
    if (shouldFailSelf) {
      if (debugTag) {
        const failReason = swipingPastEnd ? "range" : "locked";
        const failDetails = swipingPastEnd ? `${isMinIndex.value ? "min" : "max"}, ${evtTranslate}` : "";
        console.log(`${debugTag}: ${failReason} fail (${failDetails})`, evt);
        const updated = activePagers.value.slice().filter(pId => pId !== pagerId);
        activePagers.value = updated;
      }
      mgr.fail();
    } else {
      if (!activePagers.value.includes(pagerId)) {
        const updated = activePagers.value.slice();
        updated.push(pagerId);
        activePagers.value = updated;
      }
    }
  }).onUpdate(evt => {
    "worklet";

    const evtTranslate = vertical ? evt.translationY : evt.translationX;
    const crossAxisTranslate = vertical ? evt.translationX : evt.translationY;
    const isSwipingCrossAxis = Math.abs(crossAxisTranslate) > 10 && Math.abs(crossAxisTranslate) > Math.abs(evtTranslate);
    if (isGestureLocked.value || isSwipingCrossAxis) return;
    if (debugTag) {
      console.log(`${debugTag} onUpdate: ${isGestureLocked.value ? "(locked)" : ""}`, evt);
    }
    const rawVal = startTranslate.value + evtTranslate;
    const page = -rawVal / pageSize.value;
    if (page >= minIndexAnim.value && page <= maxIndexAnim.value) {
      translate.value = rawVal;
    } else {
      const pageTrans = rawVal % pageSize.value;
      const bounceTrans = pageTrans * (1 - bouncePct);
      translate.value = rawVal - bounceTrans;
    }
  }).onEnd(evt => {
    "worklet";

    const evtVelocity = vertical ? evt.velocityY : evt.velocityX;
    const evtTranslate = vertical ? evt.translationY : evt.translationX;
    const crossAxisTranslate = vertical ? evt.translationX : evt.translationY;
    const isSwipingCrossAxis = Math.abs(crossAxisTranslate) > Math.abs(evtTranslate);
    const isFling = isGestureLocked.value || isSwipingCrossAxis ? false : Math.abs(evtVelocity) > flingVelocity;
    let velocityModifier = isFling ? pageSize.value / 2 : 0;
    if (evtVelocity < 0) velocityModifier *= -1;
    let page = -1 * Math.round((translate.value + velocityModifier) / pageSize.value);
    if (page < minIndexAnim.value) page = minIndexAnim.value;
    if (page > maxIndexAnim.value) page = maxIndexAnim.value;
    const animCfg = Object.assign({}, DEFAULT_ANIMATION_CONFIG, animCfgRef.current);
    translate.value = (0, _reactNativeReanimated.withSpring)(-page * pageSize.value, animCfg);
    if (debugTag) {
      console.log(`${debugTag}: onEnd (${isGestureLocked.value ? "locked" : "unlocked"})`, evt);
    }
  }).onFinalize(evt => {
    "worklet";

    const updatedPagerIds = activePagers.value.slice().filter(id => id !== pagerId);
    activePagers.value = updatedPagerIds;
    if (debugTag) {
      console.log(`${debugTag}: onFinalize (${isGestureLocked.value ? "locked" : "unlocked"})`, evt);
    }
  }).enabled(!gesturesDisabled);
  if (typeof minDistance === "number") {
    panGesture.minDistance(minDistance);
  }
  const allGestures = (0, _react.useMemo)(() => [panGesture, ...parentGestures, ...simultaneousGestures], [panGesture, parentGestures, simultaneousGestures]);
  const wrapperStyle = (0, _react.useMemo)(() => {
    const s = {};
    if (width) s.width = width;
    if (height) s.height = height;
    return s;
  }, [width, height]);
  const gesture = (0, _react.useMemo)(() => {
    return _reactNativeGestureHandler.Gesture.Simultaneous(...allGestures);
  }, [allGestures]);
  return /*#__PURE__*/_react.default.createElement(InfinitePagerProvider, {
    simultaneousGestures: allGestures
  }, /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.GestureDetector, {
    gesture: gesture
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: [wrapperStyle, style],
    onLayout: _ref2 => {
      let {
        nativeEvent: {
          layout
        }
      } = _ref2;
      pageWidth.value = layout.width;
      pageHeight.value = layout.height;
    }
  }, pageIndices.map(pageIndex => {
    return /*#__PURE__*/_react.default.createElement(PageWrapper, {
      key: `page-provider-wrapper-${pageIndex}`,
      vertical: vertical,
      pageAnim: pageAnim,
      index: pageIndex,
      pageWidth: pageWidth,
      pageHeight: pageHeight,
      isActive: pageIndex === curIndex,
      PageComponent: PageComponent,
      renderPage: renderPage,
      style: pageWrapperStyle,
      pageInterpolatorRef: pageInterpolatorRef,
      pageBuffer: pageBuffer,
      debugTag: debugTag,
      panGesture: panGesture,
      isRootPager: !nestingDepth
    });
  }))));
}
const PageWrapper = /*#__PURE__*/_react.default.memo(_ref3 => {
  let {
    index,
    pageAnim,
    pageWidth,
    pageHeight,
    vertical,
    PageComponent,
    renderPage,
    isActive,
    style,
    pageInterpolatorRef,
    pageBuffer,
    isRootPager,
    panGesture
  } = _ref3;
  const pageSize = vertical ? pageHeight : pageWidth;
  const translation = (0, _reactNativeReanimated.useDerivedValue)(() => {
    const translateX = (index - pageAnim.value) * pageSize.value;
    return translateX;
  }, []);
  const focusAnim = (0, _reactNativeReanimated.useDerivedValue)(() => {
    if (!pageSize.value) {
      return index;
    }
    return translation.value / pageSize.value;
  }, []);
  const animStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => {
    // Short circuit page interpolation to prevent buggy initial values due to possible race condition:
    // https://github.com/software-mansion/react-native-reanimated/issues/2571
    const isInactivePageBeforeInit = index !== 0 && !pageSize.value;
    const _pageWidth = isInactivePageBeforeInit ? focusAnim : pageWidth;
    const _pageHeight = isInactivePageBeforeInit ? focusAnim : pageHeight;
    return pageInterpolatorRef.current({
      focusAnim,
      pageAnim,
      pageWidth: _pageWidth,
      pageHeight: _pageHeight,
      index,
      vertical,
      pageBuffer
    });
  }, [pageWidth, pageHeight, pageAnim, index, translation, vertical, pageBuffer]);
  if (PageComponent && renderPage) {
    console.warn("PageComponent and renderPage both defined, defaulting to PageComponent");
  }
  if (!PageComponent && !renderPage) {
    throw new Error("Either PageComponent or renderPage must be defined.");
  }
  return /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    pointerEvents: isActive ? "auto" : "none",
    style: [style, styles.pageWrapper, animStyle, isActive && styles.activePage]
  }, isRootPager && /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    pointerEvents: "none",
    style: {
      position: "absolute",
      left: -10000,
      top: -10000,
      width: 0,
      height: 0
    }
  }, /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.GestureDetector, {
    gesture: panGesture
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    collapsable: false,
    style: {
      width: 1,
      height: 1
    }
  }))), PageComponent ? /*#__PURE__*/_react.default.createElement(PageComponent, {
    index: index,
    isActive: isActive,
    focusAnim: focusAnim,
    pageWidthAnim: pageWidth,
    pageHeightAnim: pageHeight,
    pageAnim: pageAnim
  }) : renderPage === null || renderPage === void 0 ? void 0 : renderPage({
    index,
    isActive,
    focusAnim,
    pageWidthAnim: pageWidth,
    pageHeightAnim: pageHeight,
    pageAnim
  }));
});
var _default = exports.default = /*#__PURE__*/_react.default.memo( /*#__PURE__*/_react.default.forwardRef(InfinitePager));
const styles = _reactNative.StyleSheet.create({
  pageWrapper: {
    left: 0,
    right: 0,
    top: 0,
    bottom: 0,
    position: "absolute"
  },
  activePage: {
    position: "relative"
  }
});
const InfinitePagerContext = /*#__PURE__*/_react.default.createContext({
  simultaneousGestures: [],
  activePagers: (0, _reactNativeReanimated.makeMutable)([]),
  nestingDepth: 0
});
function InfinitePagerProvider(_ref4) {
  let {
    simultaneousGestures = [],
    children
  } = _ref4;
  const {
    nestingDepth,
    activePagers
  } = (0, _react.useContext)(InfinitePagerContext);
  const value = (0, _react.useMemo)(() => {
    return {
      simultaneousGestures,
      nestingDepth: nestingDepth + 1,
      activePagers
    };
  }, [simultaneousGestures, nestingDepth, activePagers]);
  return /*#__PURE__*/_react.default.createElement(InfinitePagerContext.Provider, {
    value: value
  }, children);
}
//# sourceMappingURL=index.js.map